import sys
import time
import resource
import xml.sax 
import pony

from pony import orm
from model import *


class TravahoHandler(xml.sax.ContentHandler):
    def __init__(self):
        self.movie_count = 0 
        self.movie = None
        
        self.accumulate_text = False
        self.text_accumulator = None

        self.title_region = None
        self.title_lang = None

        self.prev_movie_count = 0
        self.prev_time = time.time()
        self.movies_per_second = 0.0

    def startDocument(self):
        pass

    def startElement(self, xml_name, xml_attrs):
        if xml_name == 'movie':
            self.movie = Movie(
                title = xml_attrs['title'],
                year = xml_attrs.get('year'),
                adult = xml_attrs.get('adult',False),
                runtime = xml_attrs.get('runtime')
            )
        elif xml_name == 'genre':
            self.accumulate_text = True
            self.text_accumulator = ''
        elif xml_name == 'title':
            self.accumulate_text = True
            self.text_accumulator = ''
            self.title_region = xml_attrs.get('region')
            self.title_lang = xml_attrs.get('lang')
        elif xml_name == 'actor':
            a = Actor.get(name=xml_attrs['name'])
            if not a:
                a = Actor(name=xml_attrs['name'],
                          born=xml_attrs.get('born'),
                          died=xml_attrs.get('died'))
            try:
                Star(movie=self.movie,actor=a,role=xml_attrs.get('role'))
            except pony.orm.core.CacheIndexError:
                pass
                # print("WARNING: Ignoring duplicate cast",
                #       xml_attrs['name'], 
                #       xml_attrs['role'],
                #       self.movie.title)
        elif xml_name == 'crew-member':
            p = Person.get(name=xml_attrs['name'])
            if not p:
                p = Person(name=xml_attrs['name'],
                           born=xml_attrs.get('born'),
                           died=xml_attrs.get('died'))
            try:
                Crew(movie=self.movie,person=p,job=xml_attrs.get('job'))
            except pony.orm.core.CacheIndexError:
                pass
                # print("WARNING: Ignoring duplicate crew",
                #       xml_attrs['name'], 
                #       xml_attrs['job'],
                #       self.movie.title)
        elif xml_name == 'ratings':
            Ratings(movie=self.movie,
                    average=xml_attrs['average'],
                    votes=xml_attrs['votes'])

    def characters(self, text):
        if self.accumulate_text:
            self.text_accumulator += text

    def endElement(self, xml_name):
        if xml_name == 'movie':
            self.movie_count += 1
            self.movie_count += 1
            sys.stdout.write(" "*40+"\r %10i %3.2f mov/sec. %3.2fMB"%(self.movie_count,self.movies_per_second,resource.getrusage(resource.RUSAGE_SELF).ru_maxrss/1024/1024))

            if self.movie_count % 100 == 0:
                self.movies_per_second = (self.movie_count - self.prev_movie_count)/(time.time() - self.prev_time)
                self.prev_time = time.time()
                self.prev_movie_count = self.movie_count
            
            self.movie = None
        elif xml_name == 'genre':
            self.accumulate_text = False
            Movie_Genre(movie=self.movie,
                        genre=self.text_accumulator)
        elif xml_name == 'title':
            self.accumulate_text = False
            Movie_Title(movie=self.movie,
                        title=self.text_accumulator,
                        language=self.title_lang,
                        region=self.title_region)

    def endDocument(self):
        pass


if __name__ == '__main__':
    parser = xml.sax.make_parser()
    parser.setContentHandler(TravahoHandler())
    with orm.db_session():
        parser.parse(open('/Users/staworko/Temp/imdb.xml','r'))

