import os
import os.path
import time
import resource
import sys
import xml.sax 
import sqlite3

DB_FILE = '/Users/staworko/Temp/imdb-sql.db'

if os.path.exists(DB_FILE):
    os.remove(DB_FILE)
db = sqlite3.connect(DB_FILE)
c = db.cursor()
c.executescript(open('imdb.sql','r').read())
db.commit()



class TravahoHandler(xml.sax.ContentHandler):
    def __init__(self):
        self.movie_count = 0 
        self.movie_id = None
        
        self.accumulate_text = False
        self.text_accumulator = None

        self.title_region = None
        self.title_lang = None

        self.prev_movie_count = 0
        self.prev_time = time.time()
        self.movies_per_second = 0.0

    def startDocument(self):
        pass

    def startElement(self, xml_name, xml_attrs):
        if xml_name == 'movie':
            c.execute("INSERT INTO MOVIE(TITLE,YEAR,ADULT,RUNTIME) VALUES(?,?,?,?)",
                      (xml_attrs['title'],
                       xml_attrs.get('year'),
                       xml_attrs.get('adult'),
                       xml_attrs.get('runtime')))
            self.movie_id = c.lastrowid
        elif xml_name == 'genre':
            self.accumulate_text = True
            self.text_accumulator = ''
        elif xml_name == 'title':
            self.accumulate_text = True
            self.text_accumulator = ''
            self.title_region = xml_attrs.get('region')
            self.title_lang = xml_attrs.get('lang')
        elif xml_name == 'actor':
            c.execute("SELECT ID FROM ACTOR WHERE NAME=?",(xml_attrs['name'],))
            a = c.fetchone()
            if a:
                actor_id = a[0]
            else:
                c.execute("INSERT INTO ACTOR(NAME,BORN,DIED) VALUES (?,?,?)",
                          (xml_attrs['name'],xml_attrs.get('born'),xml_attrs.get('died')))
                actor_id = c.lastrowid
            # try:
            c.execute("INSERT INTO STAR VALUES (?,?,?)",
                      (self.movie_id,actor_id,xml_attrs.get('role')))
            # except ValueError:
            #     print("WARNING: Ignoring duplicate cast",
            #           xml_attrs['name'], 
            #           xml_attrs['role'],
            #           self.movie.title)
        elif xml_name == 'crew-member':
            c.execute("SELECT ID FROM PERSON WHERE NAME=?",(xml_attrs['name'],))
            p = c.fetchone()
            if p:
                person_id = p[0]
            else:
                c.execute("INSERT INTO PERSON(NAME,BORN,DIED) VALUES (?,?,?)",
                          (xml_attrs['name'],xml_attrs.get('born'),xml_attrs.get('died')))
                person_id = c.lastrowid
            # try:
            c.execute("INSERT INTO CREW VALUES (?,?,?)",
                      (self.movie_id,person_id,xml_attrs.get('job')))
            # except pony.orm.core.CacheIndexError:
            #     print("WARNING: Ignoring duplicate crew",
            #           xml_attrs['name'], 
            #           xml_attrs['job'],
            #           self.movie.title)
        elif xml_name == 'ratings':
            c.execute("INSERT INTO RATINGS VALUES (?,?,?)",
                      (self.movie_id,xml_attrs['average'],xml_attrs['votes']))

    def characters(self, text):
        if self.accumulate_text:
            self.text_accumulator += text

    def endElement(self, xml_name):
        if xml_name == 'movie':
            self.movie_count += 1
            sys.stdout.write(" "*40+"\r %10i %3.2f mov/sec. %3.2fMB"%(self.movie_count,self.movies_per_second,resource.getrusage(resource.RUSAGE_SELF).ru_maxrss/1024/1024))

            if self.movie_count % 10000 == 0:
                self.movies_per_second = (self.movie_count - self.prev_movie_count)/(time.time() - self.prev_time)
                self.prev_time = time.time()
                self.prev_movie_count = self.movie_count
                db.commit()

            self.movie = None
        elif xml_name == 'genre':
            self.accumulate_text = False
            c.execute("INSERT INTO MOVIE_GENRE VALUES(?,?)",
                      (self.movie_id,self.text_accumulator))
        elif xml_name == 'title':
            self.accumulate_text = False
            c.execute("INSERT INTO TITLES VALUES(?,?,?,?)",
                      (self.movie_id,
                       self.text_accumulator,
                       self.title_lang,
                       self.title_region))

    def endDocument(self):
        pass


if __name__ == '__main__':
    parser = xml.sax.make_parser()
    parser.setContentHandler(TravahoHandler())
    parser.parse(open('/Users/staworko/Temp/imdb.xml','r'))
    #parser.parse(open('imdb.xml','r'))

